import { Injectable, Directive, ElementRef, HostListener, Input, EventEmitter, Output, NgModule, defineInjectable } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class NgxDragDropService {
    constructor() {
        this.defaultInstanceName = 'ngx-dragdrop';
        this.setDefaultInstance();
    }
    /**
     * @return {?}
     */
    get instanceName() {
        return this.name;
    }
    /**
     * @param {?} name
     * @return {?}
     */
    set instanceName(name) {
        this.name = name;
    }
    /**
     * @return {?}
     */
    setDefaultInstance() {
        this.name = this.defaultInstanceName;
    }
    /**
     * @return {?}
     */
    isDefaultInstance() {
        return this.name === this.defaultInstanceName;
    }
}
NgxDragDropService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
NgxDragDropService.ctorParameters = () => [];
/** @nocollapse */ NgxDragDropService.ngInjectableDef = defineInjectable({ factory: function NgxDragDropService_Factory() { return new NgxDragDropService(); }, token: NgxDragDropService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class NgxDraggableDirective {
    /**
     * @param {?} el
     * @param {?} dragDropService
     */
    constructor(el, dragDropService) {
        this.el = el;
        this.dragDropService = dragDropService;
        this._isDraggable = false;
    }
    /**
     * @param {?} dragEnabled
     * @return {?}
     */
    set ngxDraggable(dragEnabled) {
        /** @type {?} */
        const draggable = !!dragEnabled;
        this.el.nativeElement.setAttribute('draggable', draggable);
        this._isDraggable = draggable;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragStart(event) {
        if (this.el.nativeElement === event.target) {
            event.target.classList.add(this.ngxDraggedClass);
            event.dataTransfer.setData('data', JSON.stringify({ dd: this.ngxDragData }));
            if (this.ngxDragTag) {
                this.dragDropService.instanceName = this.ngxDragTag;
            }
            else {
                this.dragDropService.setDefaultInstance();
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragEnd(event) {
        if (this.el.nativeElement === event.target) {
            event.target.classList.remove(this.ngxDraggedClass);
        }
    }
}
NgxDraggableDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngxDraggable]'
            },] }
];
/** @nocollapse */
NgxDraggableDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: NgxDragDropService }
];
NgxDraggableDirective.propDecorators = {
    ngxDraggable: [{ type: Input }],
    ngxDragTag: [{ type: Input }],
    ngxDraggedClass: [{ type: Input }],
    ngxDragData: [{ type: Input }],
    onDragStart: [{ type: HostListener, args: ['dragstart', ['$event'],] }],
    onDragEnd: [{ type: HostListener, args: ['dragend', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class NgxDroppableDirective {
    /**
     * @param {?} el
     * @param {?} dragDropService
     */
    constructor(el, dragDropService) {
        this.el = el;
        this.dragDropService = dragDropService;
        this.ngxDroppable = new EventEmitter();
        this.ngxDropTags = [this.dragDropService.instanceName];
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragEnter(event) {
        if (event.preventDefault) {
            event.preventDefault();
        }
        if (event.stopPropagation) {
            event.stopPropagation();
        }
        if (this.el.nativeElement === event.target) {
            if (this.ngxDropTags.indexOf(this.dragDropService.instanceName) !== -1) {
                event.target.classList.add(this.dragDropService.instanceName);
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragOver(event) {
        if (event.preventDefault) {
            event.preventDefault();
        }
        if (event.stopPropagation) {
            event.stopPropagation();
        }
        if (this.el.nativeElement === event.target) {
            if (this.ngxDropTags.indexOf(this.dragDropService.instanceName) !== -1) {
                event.dataTransfer.effectAllowed = 'all';
                event.dataTransfer.dropEffect = 'move';
            }
            else {
                event.dataTransfer.effectAllowed = 'none';
                event.dataTransfer.dropEffect = 'none';
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragLeave(event) {
        if (event.preventDefault) {
            event.preventDefault();
        }
        if (event.stopPropagation) {
            event.stopPropagation();
        }
        if (this.el.nativeElement === event.target) {
            if (this.ngxDropTags.indexOf(this.dragDropService.instanceName) !== -1) {
                event.target.classList.remove(this.dragDropService.instanceName);
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDrop(event) {
        if (event.preventDefault) {
            event.preventDefault();
        }
        if (event.stopPropagation) {
            event.stopPropagation();
        }
        if (this.el.nativeElement === event.target) {
            /** @type {?} */
            const dragTag = this.dragDropService.instanceName;
            /** @type {?} */
            const data = event.dataTransfer.getData('data');
            /** @type {?} */
            let dragData;
            try {
                dragData = JSON.parse(data).dd;
                if (this.ngxDropTags.indexOf(this.dragDropService.instanceName) !== -1) {
                    event.target.classList.remove(this.dragDropService.instanceName);
                    this.ngxDroppable.emit({ dragTag, dragData: dragData, dropData: this.ngxDropData });
                }
            }
            catch (e) {
                if (console && typeof console.log === 'function') {
                    console.log(e);
                    console.log('Drop failed! Data: ', data);
                }
            }
        }
    }
}
NgxDroppableDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngxDroppable]'
            },] }
];
/** @nocollapse */
NgxDroppableDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: NgxDragDropService }
];
NgxDroppableDirective.propDecorators = {
    ngxDroppable: [{ type: Output }],
    ngxDropTags: [{ type: Input }],
    ngxDropData: [{ type: Input }],
    onDragEnter: [{ type: HostListener, args: ['dragenter', ['$event'],] }],
    onDragOver: [{ type: HostListener, args: ['dragover', ['$event'],] }],
    onDragLeave: [{ type: HostListener, args: ['dragleave', ['$event'],] }],
    onDrop: [{ type: HostListener, args: ['drop', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class NgxDragDropModule {
}
NgxDragDropModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                declarations: [
                    NgxDraggableDirective,
                    NgxDroppableDirective
                ],
                exports: [
                    NgxDraggableDirective,
                    NgxDroppableDirective
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */

export { NgxDragDropService, NgxDraggableDirective, NgxDroppableDirective, NgxDragDropModule };

//# sourceMappingURL=ngx-dragdrop.js.map